package com.diagbot.aggregate;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.ResultInfo;
import com.diagbot.entity.ResultRule;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.facade.MedRuleConvertFacade;
import com.diagbot.facade.MedSearchDataFacade;
import com.diagbot.facade.RunningInfoFacade;
import com.diagbot.service.ResultInfoService;
import com.diagbot.service.ResultRuleService;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.FastJsonUtils;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.RuleTestVO;
import com.diagbot.vo.RunningInfoVO;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/1/20 15:00
 */
@Component
@Slf4j
public class RuleTestAggregate {
    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    @Qualifier("resultRuleServiceImpl")
    private ResultRuleService resultRuleService;
    @Autowired
    @Qualifier("resultInfoServiceImpl")
    private ResultInfoService resultInfoService;
    @Autowired
    private RunningInfoFacade runningInfoFacade;
    @Autowired
    private MedRuleConvertFacade medRuleConvertFacade;
    @Autowired
    private MedSearchDataFacade medSearchDataFacade;

    @DataProvider("ruleTest")
    public Boolean ruleTest(
            @DataConsumer("billRuleTest") Boolean billRuleTest,
            @DataConsumer("billTransfusionRuleTest") Boolean billTransfusionRuleTest,
            @DataConsumer("criticalLisRuleTest") Boolean criticalLisRuleTest,
            @DataConsumer("criticalPacsRuleTest") Boolean criticalPacsRuleTest,
            @DataConsumer("highriskDrugRuleTest") Boolean highriskDrugRuleTest,
            @DataConsumer("highriskOperationRuleTest") Boolean highriskOperationRuleTest,
            @DataConsumer("otherLisRuleTest") Boolean otherLisRuleTest,
            @DataConsumer("otherPacsRuleTest") Boolean otherPacsRuleTest,
            @DataConsumer("otherTransfusionRuleTest") Boolean otherTransfusionRuleTest,
            @DataConsumer("reBillRuleTest") Boolean reBillRuleTest) {

        return billRuleTest
                && billTransfusionRuleTest
                && criticalLisRuleTest
                && criticalPacsRuleTest
                && highriskDrugRuleTest
                && highriskOperationRuleTest
                && otherLisRuleTest
                && otherPacsRuleTest
                && otherTransfusionRuleTest
                && reBillRuleTest;
    }

    /**
     * 开单合理性测试
     *
     * @param ruleTestVO
     */
    @DataProvider("billRuleTest")
    public Boolean billRuleTest(@InvokeParameter("billRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 开单合理性测试-输血
     *
     * @param ruleTestVO
     */
    @DataProvider("billTransfusionRuleTest")
    public Boolean billTransfusionRuleTest(@InvokeParameter("billTransfusionRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 危急值规则测试-检验
     *
     * @param ruleTestVO
     */
    @DataProvider("criticalLisRuleTest")
    public Boolean criticalLisRuleTest(@InvokeParameter("criticalLisRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 危急值规则测试-检查
     *
     * @param ruleTestVO
     */
    @DataProvider("criticalPacsRuleTest")
    public Boolean criticalPacsRuleTest(@InvokeParameter("criticalPacsRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 高危药品测试
     *
     * @return
     */
    @DataProvider("highriskDrugRuleTest")
    public Boolean highriskDrugRuleTest(@InvokeParameter("highriskDrugRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 高危手术测试
     *
     * @return
     */
    @DataProvider("highriskOperationRuleTest")
    public Boolean highriskOperationRuleTest(@InvokeParameter("highriskOperationRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }


    /**
     * 其他值提醒-化验
     *
     * @param ruleTestVO
     * @return
     */
    @DataProvider("otherLisRuleTest")
    public Boolean otherLisRuleTest(@InvokeParameter("otherLisRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 其他值提醒-辅检
     *
     * @param ruleTestVO
     * @return
     */
    @DataProvider("otherPacsRuleTest")
    public Boolean otherPacsRuleTest(@InvokeParameter("otherPacsRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }


    /**
     * 其他值提醒-输血
     *
     * @param ruleTestVO
     * @return
     */
    @DataProvider("otherTransfusionRuleTest")
    public Boolean otherTransfusionRuleTest(@InvokeParameter("otherTransfusionRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }

    /**
     * 正常项目重复开单
     *
     * @param ruleTestVO
     * @return
     */
    @DataProvider("reBillRuleTest")
    public Boolean reBillRuleTest(@InvokeParameter("reBillRuleVO") RuleTestVO ruleTestVO) {
        billTestExe(ruleTestVO);
        return true;
    }


    /**
     * 执行测试
     *
     * @param ruleTestVO
     * @return
     */
    public Boolean billTestExe(RuleTestVO ruleTestVO) {
        Long hospitalId = ruleTestVO.getHospitalId();
        //开始运行
        RunningInfoVO runningInfoVO = new RunningInfoVO();
        runningInfoVO.setCaseId(ruleTestVO.getCaseId());
        runningInfoVO.setHospitalId(ruleTestVO.getHospitalId());
        runningInfoVO.setStatus(1);
        runningInfoFacade.updateStatus(runningInfoVO);

        //生成测试规则
        List<ResultRule> resultList = medRuleConvertFacade.billConvert(hospitalId, ruleTestVO.getCaseId());
        if (ListUtil.isEmpty(resultList)) {
            return true;
        }
        //远程调用推理
        List<ResultRule> saveList = callIndication(resultList, hospitalId);
        //保存测试结果
        saveResultRule(saveList, ruleTestVO);

        //结束运行
        runningInfoVO.setStatus(0);
        runningInfoFacade.updateStatus(runningInfoVO);
        return true;
    }


    /**
     * 开单合理性远程调用接口
     *
     * @param list
     * @param hospitalId
     * @return
     */
    public List<ResultRule> callIndication(List<ResultRule> list, Long hospitalId) {
        List<ResultRule> retList = Lists.newLinkedList();
        if (ListUtil.isEmpty(list)) {
            return retList;
        }
        Map<Long, List<ResultRule>> ruleMap = list
                .stream()
                .collect(Collectors.groupingBy(ResultRule::getRuleOrderNo));
        for (Map.Entry<Long, List<ResultRule>> entry : ruleMap.entrySet()) {

            if (entry.getValue().get(0).getSuccess() != null
                    && entry.getValue().get(0).getSuccess().equals(0)) {
                retList.addAll(entry.getValue());
                continue;
            }

            Integer ruleType = entry.getValue().get(0).getRuleType();
            IndicationPushVO indicationPushVO = medSearchDataFacade.createBillVO(entry.getValue(),hospitalId);

            //入参
            entry.getValue().forEach(result -> {
                result.setInputParams(FastJsonUtils.getBeanToJson(indicationPushVO));
            });

            RespDTO<IndicationDTO> respDTO = cdssCoreClient.indication(indicationPushVO);
            if (RespDTOUtil.respIsOK(respDTO)) {
                IndicationDTO data = respDTO.data;
                //出参
                entry.getValue().forEach(result -> {
                    result.setOutputParams(FastJsonUtils.getBeanToJson(respDTO.data));
                });
                List<BillMsg> msgList = Lists.newLinkedList();
                switch (ruleType) {
                    case 1:
                    case 6:
                        msgList = data.getBillMsgList();
                        break;
                    case 2:
                        msgList = data.getHighRiskList();
                        break;
                    case 3:
                        msgList = data.getCriticalValList();
                        break;
                    case 4:
                    case 5:
                        msgList = data.getOtherList();
                        break;
                    default:
                        break;
                }

                if (ListUtil.isEmpty(msgList)) {
                    entry.getValue().forEach(result -> {
                        result.setSuccess(0);
                        result.setMessage("提醒信息未出现");
                        retList.add(result);
                    });
                    continue;
                }

                Boolean success = false;
                for (BillMsg msg : msgList) {
                    if (!msg.getMsg().equals(entry.getValue().get(0).getExpectedOutput())) {
                        continue;
                    } else {
                        success = true;
                    }
                    for (ResultRule result : entry.getValue()) {
                        ResultRule saveResult = new ResultRule();
                        BeanUtil.copyProperties(result, saveResult);
                        saveResult.setOutput(msg.getMsg());
                        if (saveResult.getOutput().equals(saveResult.getExpectedOutput())) {
                            saveResult.setSuccess(1);
                        } else {
                            saveResult.setSuccess(0);
                            saveResult.setMessage("提示内容与预期不符");
                        }
                        retList.add(saveResult);
                    }
                }
                if (!success) {
                    for (ResultRule result : entry.getValue()) {
                        Integer count = 1;
                        result.setSuccess(0);
                        result.setMessage(result.getMessage() + "提示内容（"
                                + (count++) + "、"
                                + msgList.stream().map(i -> i.getMsg()).collect(Collectors.joining("；" + (count++) + "、"))
                                + "）与预期不符");
                        retList.add(result);
                    }
                }
            } else {
                for (ResultRule result : entry.getValue()) {
                    result.setInputParams(FastJsonUtils.getBeanToJson(indicationPushVO));
                    result.setSuccess(0);
                    result.setMessage("远程调用推理接口失败");
                    retList.add(result);
                }
            }
        }
        return retList;
    }

    /**
     * 保存开单合理性测试结果
     *
     * @param list
     * @param ruleTestVO
     * @return
     */
    @Transactional
    public Boolean saveResultRule(List<ResultRule> list, RuleTestVO ruleTestVO) {
        Date now = new Date();
        Integer totleNum = list.stream()
                .map(i -> i.getRuleOrderNo())
                .distinct()
                .collect(Collectors.toList()).size();
        Integer successNum = list.stream()
                .filter(i -> i.getSuccess().equals(1))
                .map(i -> i.getRuleOrderNo())
                .distinct()
                .collect(Collectors.toList()).size();
        Integer failedNum = list.stream().filter(i -> i.getSuccess().equals(0))
                .map(i -> i.getRuleOrderNo())
                .distinct()
                .collect(Collectors.toList()).size();
        QueryWrapper<ResultInfo> resultInfoQueryWrapper = new QueryWrapper<>();
        resultInfoQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", ruleTestVO.getHospitalId())
                .eq("case_id", ruleTestVO.getCaseId());
        ResultInfo resultInfo = resultInfoService.getOne(resultInfoQueryWrapper, false);
        if (resultInfo == null) {
            resultInfo = new ResultInfo();
            resultInfo.setHospitalId(ruleTestVO.getHospitalId());
            resultInfo.setCaseId(ruleTestVO.getCaseId());
            resultInfo.setGmtCreate(now);
        }
        resultInfo.setGmtModified(now);
        resultInfo.setRuleNum(totleNum);
        resultInfo.setRuleSuccessNum(successNum);
        resultInfo.setRuleFailedNum(failedNum);
        resultInfoService.saveOrUpdate(resultInfo);

        QueryWrapper<ResultRule> resultRuleQueryWrapper = new QueryWrapper<>();
        resultRuleQueryWrapper.eq("result_id", resultInfo.getId());
        resultRuleService.remove(resultRuleQueryWrapper);

        for (ResultRule resultRule : list) {
            resultRule.setGmtCreate(now);
            resultRule.setGmtModified(now);
            resultRule.setResultId(resultInfo.getId());
        }

        resultRuleService.saveOrUpdateBatch(list);
        return true;
    }
}